@@ -615,16 +615,18 @@ sub _generate_operator {
     $self->_error("Invalid expression", $node);
 }
 
-sub _can_print_optimize {
+sub _can_optimize_print {
     my($self, $name, $node) = @_;
 
     return 0 if !$OPTIMIZE;
     return 0 if !($name eq 'print' or $name eq 'print_raw');
 
+    my $maybe_name = $node->first;
     return $node->arity eq 'call'
-        && $node->first->arity eq 'name'
+        && $maybe_name->arity eq 'name'
         && @{$node->second} == 1 # args of the filter
-        && any_in($node->first->id, qw(raw mark_raw html));
+        && any_in($maybe_name->id, qw(raw mark_raw html))
+        && !$self->overridden_builtin->{$maybe_name->id};
 }
 
 # also deal with smart escaping
@@ -639,7 +641,8 @@ sub _generate_print {
     }
 
     foreach my $arg(@{ $node->first }){
-        if( $self->overridden_builtin->{html_escape} ) {
+        if( $proc eq 'print' && $self->overridden_builtin->{html_escape} ) {
+            # default behaviour of print() is overridden
             push @code,
                 $self->opcode('pushmark'),
                 $self->compile_ast($arg),
@@ -654,7 +657,7 @@ sub _generate_print {
                 $self->opcode( $proc . '_s' => $arg->value,
                                line         => $arg->line );
         }
-        elsif($self->_can_print_optimize($proc, $arg)){
+        elsif($self->_can_optimize_print($proc, $arg)){
             my $filter      = $arg->first;
             my $filter_name = $filter->id;
             my $command = $builtin{ $filter_name }[0] eq 'builtin_mark_raw'
@@ -763,8 +766,8 @@ sub _generate_for {
     if(@{$vars} != 1) {
         $self->_error("A for-loop requires single variable for each item", $node);
     }
-    local $self->{lvar}  = { %{$self->lvar} }; # new scope
-    local $self->{const} = [];                 # new scope
+    local $self->{lvar}  = { %{$self->lvar} };  # new scope
+    local $self->{const} = [ @{$self->const} ]; # new scope
     local $self->{in_loop} = _FOR_LOOP;
 
     my @code = $self->compile_ast($expr);
@@ -1083,6 +1086,11 @@ sub _generate_nil {
     return $self->opcode('nil');
 }
 
+sub _generate_vars {
+    my($self) = @_;
+    return $self->opcode('vars');
+}
+
 sub _generate_composer {
     my($self, $node) = @_;
 
@@ -1231,15 +1239,13 @@ sub _generate_call {
     my $callable = $node->first; # function or macro
     my $args     = $node->second;
 
-    if(my $intern = $builtin{$callable->id}) {
+    if(my $intern = $builtin{$callable->id} and !$self->overridden_builtin->{$callable->id}) {
         if(@{$args} != 1) {
             $self->_error("Wrong number of arguments for $callable", $node);
         }
 
-        if( !$self->overridden_builtin->{ $intern->[0] } ) {
-            return $self->compile_ast($args->[0]),
-                [ $intern->[0] => undef, $node->line ];
-        }
+        return $self->compile_ast($args->[0]),
+            [ $intern->[0] => undef, $node->line ];
     }
 
     return(
@@ -1327,7 +1333,8 @@ sub _generate_constant {
     $self->{lvar_id}    = $self->lvar_use(1); # don't use local()
 
     if($OPTIMIZE) {
-        if(@expr == 1 && any_in($expr[0][_OP_NAME], qw(literal load_lvar))) {
+        if(@expr == 1
+                && any_in($expr[0][_OP_NAME], qw(literal load_lvar))) {
             $expr[0][_OP_COMMENT] = "constant $lvar_name";
             $self->const->[$lvar_id] = \@expr;
             return @expr; # no real definition
